#pragma once

#include <vector>

#include "glm/glm.hpp"

struct Symbol
{
    enum Operation
    {
        IGN = 0,
        OCCUPIED,
        UNOCCUPIED,
        EQ,
        NEQ,
        LT,
        LEQ,
        GT,
        GEQ,
    };

    std::vector<std::pair<Operation, float>> ops;
    glm::ivec3 dim;
};

struct Transform
{
    enum Operation
    {
        NOP = 0,
        SET = 1,
        ADD = 2,
    };

    std::vector<std::pair<Operation, float>> ops;
    glm::ivec3 dim;
};

class Grammar
{
    public:
        unsigned int addSymbol(const Symbol& symbol);
        unsigned int addTransform(const Transform& transform);

        void addRule(unsigned int symbolIndex, unsigned int transformIndex);

        // Uses a linear array of values to represent a voxel grid
        void parse(std::vector<float>& voxelGrid, const glm::ivec3& gridDim, unsigned int iterations, const glm::ivec3& stride);

    private:
        bool checkSymbolMatch(const Symbol& s, std::vector<float>& voxelGrid, const glm::ivec3& voxelGridIndex, const glm::ivec3& voxelGridDim);
        void transform(const Transform& t, std::vector<float>& voxelGrid, const glm::ivec3& voxelGridIndex, const glm::ivec3& voxelGridDim);

    private:
        std::unordered_map<unsigned int, std::vector<unsigned int>> ruleLookup;
        std::vector<Symbol> symbols;
        std::vector<Transform> transforms;
};